@@ -8,7 +8,7 @@ sub run {
     my $memd = test_client(protocol_class => $protocol, selector_class => $selector);
 
     my $bogus_server = 'you.should.not.be.able.to.connect.to.me:11211';
-    push @{$memd->{servers}}, $bogus_server;
+    $memd->add_server( $bogus_server );
 
     my $key_base = "BigF*ckingTruckHitMe";
     my $value = join('.', time(), $$, {}, rand());
@@ -17,15 +17,21 @@ sub run {
 
     my $warn_called = 0;
     local $SIG{__WARN__} = sub {
-        like( $_[0], qr/^failed to connect to $bogus_server/);
+#        like( $_[0], qr/^failed to connect to $bogus_server/);
         $warn_called++;
     };
 
     foreach my $i (1..50) {
         my $key = $key_base . $i;
         $cv->begin;
-        $memd->set($key, $value, sub {});
-        $memd->get($key, sub { is ($_[0], $value, "values match"); $cv->end });
+        $memd->set($key, $value, sub {
+            if (ok($_[0], "set $key works")) {
+                $memd->get($key, sub {
+                    is ($_[0], $value, "values match for $key");
+                    $cv->end;
+                });
+            }
+        });
     }
 
     $cv->recv;
